@@ -30,6 +30,8 @@
 #include "gitg-commit.h"
 #include "gitg-utils.h"
 #include "gitg-diff-view.h"
+#include "gitg-preferences.h"
+#include "gitg-data-binding.h"
 
 #define GITG_COMMIT_VIEW_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE((object), GITG_TYPE_COMMIT_VIEW, GitgCommitViewPrivate))
 #define CATEGORY_UNSTAGE_HUNK "CategoryUnstageHunk"
@@ -70,6 +72,7 @@ struct _GitgCommitViewPrivate
 	GtkSourceView *changes_view;
 	GtkTextView *comment_view;
 	GtkCheckButton *check_button_signed_off_by;
+	GtkCheckButton *check_button_amend;
 	
 	GtkHScale *hscale_context;
 	gint context_size;
@@ -91,7 +94,7 @@ struct _GitgCommitViewPrivate
 
 static void gitg_commit_view_buildable_iface_init(GtkBuildableIface *iface);
 
-G_DEFINE_TYPE_EXTENDED(GitgCommitView, gitg_commit_view, GTK_TYPE_HPANED, 0,
+G_DEFINE_TYPE_EXTENDED(GitgCommitView, gitg_commit_view, GTK_TYPE_VPANED, 0,
 	G_IMPLEMENT_INTERFACE(GTK_TYPE_BUILDABLE, gitg_commit_view_buildable_iface_init));
 
 static GtkBuildableIface parent_iface;
@@ -121,6 +124,7 @@ static void on_revert_changes(GtkAction *action, GitgCommitView *view);
 static void on_ignore_file(GtkAction *action, GitgCommitView *view);
 static void on_unstage_changes(GtkAction *action, GitgCommitView *view);
 
+static void on_check_button_amend_toggled (GtkToggleButton *button, GitgCommitView *view);
 
 static void
 gitg_commit_view_finalize(GObject *object)
@@ -777,8 +781,6 @@ on_tree_view_drag_data_get (GtkWidget        *widget,
                             guint             time,
                             GitgCommitView   *view)
 {
-	GtkTreeSelection *sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
-	GtkTreeModel *model;
 	GList *selected;
 	GList *item;
 	gchar **uris;
@@ -818,7 +820,6 @@ on_tree_view_staged_drag_data_received(GtkWidget        *widget,
 	/* Stage all the files dropped on this */
 	gchar **uris = gtk_selection_data_get_uris(data);
 	gchar **uri;
-	GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
 
 	for (uri = uris; *uri; ++uri)
 	{
@@ -913,7 +914,18 @@ gitg_commit_view_parser_finished(GtkBuildable *buildable, GtkBuilder *builder)
 	/* Store widgets */
 	GitgCommitView *self = GITG_COMMIT_VIEW(buildable);
 	
-	self->priv->ui_manager = g_object_ref(gtk_builder_get_object(builder, "uiman"));
+	GtkBuilder *b = gitg_utils_new_builder("gitg-commit-menu.ui");
+	self->priv->ui_manager = g_object_ref(gtk_builder_get_object(b, "uiman"));
+
+	g_signal_connect(gtk_builder_get_object(b, "StageChangesAction"), "activate", G_CALLBACK(on_stage_changes), self);
+	g_signal_connect(gtk_builder_get_object(b, "RevertChangesAction"), "activate", G_CALLBACK(on_revert_changes), self);
+	g_signal_connect(gtk_builder_get_object(b, "IgnoreFileAction"), "activate", G_CALLBACK(on_ignore_file), self);
+	g_signal_connect(gtk_builder_get_object(b, "UnstageChangesAction"), "activate", G_CALLBACK(on_unstage_changes), self);
+
+	self->priv->group_context = GTK_ACTION_GROUP(gtk_builder_get_object(b, "action_group_commit_context"));
+	
+	g_object_unref(b);
+	
 	self->priv->tree_view_unstaged = GTK_TREE_VIEW(gtk_builder_get_object(builder, "tree_view_unstaged"));
 	self->priv->tree_view_staged = GTK_TREE_VIEW(gtk_builder_get_object(builder, "tree_view_staged"));
 	
@@ -926,9 +938,18 @@ gitg_commit_view_parser_finished(GtkBuildable *buildable, GtkBuilder *builder)
 	self->priv->changes_view = GTK_SOURCE_VIEW(gtk_builder_get_object(builder, "source_view_changes"));
 	self->priv->comment_view = GTK_TEXT_VIEW(gtk_builder_get_object(builder, "text_view_comment"));
 	self->priv->check_button_signed_off_by = GTK_CHECK_BUTTON(gtk_builder_get_object(builder, "check_button_signed_off_by"));
+	self->priv->check_button_amend = GTK_CHECK_BUTTON(gtk_builder_get_object(builder, "check_button_amend"));
+	
+	GitgPreferences *preferences = gitg_preferences_get_default();
+	
+	gitg_data_binding_new(preferences, "message-show-right-margin",
+	                      self->priv->comment_view, "show-right-margin");
+
+	gitg_data_binding_new(preferences, "message-right-margin-at",
+	                      self->priv->comment_view, "right-margin-position");
 	
 	self->priv->hscale_context = GTK_HSCALE(gtk_builder_get_object(builder, "hscale_context"));
-	self->priv->group_context = GTK_ACTION_GROUP(gtk_builder_get_object(builder, "action_group_commit_context"));
+	gtk_range_set_value (GTK_RANGE (self->priv->hscale_context), 3);
 	
 	initialize_dnd_staged(self);
 	initialize_dnd_unstaged(self);
@@ -987,10 +1008,10 @@ gitg_commit_view_parser_finished(GtkBuildable *buildable, GtkBuilder *builder)
 	
 	g_signal_connect(self->priv->hscale_context, "value-changed", G_CALLBACK(on_context_value_changed), self);
 	
-	g_signal_connect(gtk_builder_get_object(builder, "StageChangesAction"), "activate", G_CALLBACK(on_stage_changes), self);
-	g_signal_connect(gtk_builder_get_object(builder, "RevertChangesAction"), "activate", G_CALLBACK(on_revert_changes), self);
-	g_signal_connect(gtk_builder_get_object(builder, "IgnoreFileAction"), "activate", G_CALLBACK(on_ignore_file), self);
-	g_signal_connect(gtk_builder_get_object(builder, "UnstageChangesAction"), "activate", G_CALLBACK(on_unstage_changes), self);
+	g_signal_connect (self->priv->check_button_amend,
+	                  "toggled",
+	                  G_CALLBACK (on_check_button_amend_toggled),
+	                  self);
 }
 
 static void
@@ -1420,10 +1441,11 @@ on_commit_clicked(GtkButton *button, GitgCommitView *view)
 	}
 	
 	gboolean signoff = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(view->priv->check_button_signed_off_by));
-	
+	gboolean amend = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (view->priv->check_button_amend));
+
 	GError *error = NULL;
 	
-	if (!gitg_commit_commit(view->priv->commit, comment, signoff, &error))
+	if (!gitg_commit_commit(view->priv->commit, comment, signoff, amend, &error))
 	{
 		if (error && error->domain == GITG_COMMIT_ERROR && error->code == GITG_COMMIT_ERROR_SIGNOFF)
 			show_error(view, _("Your user name or email could not be retrieved for use in the sign off message"));
@@ -1436,6 +1458,8 @@ on_commit_clicked(GtkButton *button, GitgCommitView *view)
 	else
 	{
 		gtk_text_buffer_set_text(gtk_text_view_get_buffer(view->priv->comment_view), "", -1);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (view->priv->check_button_amend), FALSE);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (view->priv->check_button_signed_off_by), FALSE);
 	}
 	
 	g_free(comment);
@@ -1668,3 +1692,27 @@ on_changes_view_popup_menu(GtkTextView *textview, GtkMenu *menu, GitgCommitView
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu), revert);
 	}
 }
+
+static void
+on_check_button_amend_toggled (GtkToggleButton *button, GitgCommitView *view)
+{
+	gboolean active = gtk_toggle_button_get_active (button);
+	GtkTextBuffer *buffer = gtk_text_view_get_buffer (view->priv->comment_view);
+	GtkTextIter start;
+	GtkTextIter end;
+	
+	gtk_text_buffer_get_bounds (buffer, &start, &end);
+	
+	if (active && gtk_text_iter_compare (&start, &end) == 0)
+	{
+		// Get last commit message
+		gchar *message = gitg_commit_amend_message (view->priv->commit);
+		
+		if (message)
+		{
+			gtk_text_buffer_set_text (buffer, message, -1);
+		}
+		
+		g_free (message);
+	}
+}
