@@ -345,13 +345,13 @@ delete_file(GFile *key, GitgChangedFile *value, GitgCommit *commit)
 }
 
 static void
-refresh_done(GitgRunner *runner, GitgCommit *commit)
+refresh_done(GitgRunner *runner, gboolean cancelled, GitgCommit *commit)
 {
 	g_hash_table_foreach_remove(commit->priv->files, (GHRFunc)delete_file, commit);
 }
 
 static void
-read_unstaged_files_end(GitgRunner *runner, GitgCommit *commit)
+read_unstaged_files_end(GitgRunner *runner, gboolean cancelled, GitgCommit *commit)
 {
 	gchar *head = gitg_repository_parse_head(commit->priv->repository);
 	gitg_runner_cancel(runner);
@@ -368,7 +368,7 @@ read_unstaged_files_update(GitgRunner *runner, gchar **buffer, GitgCommit *commi
 }
 
 static void
-read_other_files_end(GitgRunner *runner, GitgCommit *commit)
+read_other_files_end(GitgRunner *runner, gboolean cancelled, GitgCommit *commit)
 {
 	gitg_runner_cancel(runner);
 	
@@ -420,7 +420,7 @@ read_other_files_update(GitgRunner *runner, gchar **buffer, GitgCommit *commit)
 }
 
 static void
-update_index_end(GitgRunner *runner, GitgCommit *commit)
+update_index_end(GitgRunner *runner, gboolean cancelled, GitgCommit *commit)
 {
 	gitg_runner_cancel(runner);
 	runner_connect(commit, G_CALLBACK(read_other_files_update), G_CALLBACK(read_other_files_end));
@@ -455,7 +455,7 @@ gitg_commit_refresh(GitgCommit *commit)
 	if (commit->priv->repository)
 		update_index(commit);
 	else
-		refresh_done(commit->priv->runner, commit);
+		refresh_done(commit->priv->runner, FALSE, commit);
 }
 
 static void
@@ -540,13 +540,14 @@ refresh_changes(GitgCommit *commit, GitgChangedFile *file)
 	update_index_unstaged(commit, file);
 	
 	GitgChangedFileChanges changes = gitg_changed_file_get_changes(file);
-	
-	if (changes == GITG_CHANGED_FILE_CHANGES_NONE)
+	GitgChangedFileStatus status = gitg_changed_file_get_status(file);
+
+	if (changes == GITG_CHANGED_FILE_CHANGES_NONE && status == GITG_CHANGED_FILE_CHANGES_NONE)
 		gitg_changed_file_set_status(file, GITG_CHANGED_FILE_STATUS_NEW);
 	else if ((changes & GITG_CHANGED_FILE_CHANGES_CACHED) && (changes & GITG_CHANGED_FILE_CHANGES_UNSTAGED))
 		gitg_changed_file_set_status(file, GITG_CHANGED_FILE_STATUS_MODIFIED);
 	
-	if (gitg_changed_file_get_status(file) == GITG_CHANGED_FILE_STATUS_NEW &&
+	if (status == GITG_CHANGED_FILE_STATUS_NEW &&
 	    !(changes & GITG_CHANGED_FILE_CHANGES_CACHED))
 	{
 		gitg_changed_file_set_changes(file, GITG_CHANGED_FILE_CHANGES_UNSTAGED);
@@ -709,8 +710,69 @@ get_signed_off_line(GitgCommit *commit)
 	return ret;
 }
 
+static void
+on_commit_tree_update (GitgRunner *runner, gchar **lines, GString *buffer)
+{
+	while (lines && *lines)
+	{
+		if (buffer->len != 0)
+		{
+			g_string_append_c (buffer, '\n');
+		}
+		
+		g_string_append (buffer, *lines);
+		++lines;
+	}
+}
+
+static void
+set_amend_environment (GitgCommit *commit, GitgRunner *runner)
+{
+	gchar **out;
+	
+	out = gitg_repository_command_with_outputv (commit->priv->repository,
+	                                            NULL,
+	                                            "cat-file",
+	                                            "commit",
+	                                            "HEAD",
+	                                            NULL);
+
+	// Parse author
+	GRegex *r = g_regex_new ("^author (.*) <([^>]*)> ([0-9]+.*)$",
+	                         G_REGEX_CASELESS,
+	                         0,
+	                         NULL);
+
+	GMatchInfo *info = NULL;
+	gchar **ptr = out;
+	
+	while (ptr && *ptr)
+	{
+		if (g_regex_match (r, *ptr, 0, &info))
+		{
+			gchar *name = g_match_info_fetch (info, 1);
+			gchar *email = g_match_info_fetch (info, 2);
+			gchar *date = g_match_info_fetch (info, 3);
+
+			gitg_runner_add_environment (runner, "GIT_AUTHOR_NAME", name);
+			gitg_runner_add_environment (runner, "GIT_AUTHOR_EMAIL", email);
+			gitg_runner_add_environment (runner, "GIT_AUTHOR_DATE", date);
+			
+			g_free (name);
+			g_free (email);
+			g_free (date);
+		
+			break;
+		}
+		
+		++ptr;
+	}
+	
+	g_strfreev (out);
+}
+
 static gboolean 
-commit_tree(GitgCommit *commit, gchar const *tree, gchar const *comment, gboolean signoff, gchar **ref, GError **error)
+commit_tree(GitgCommit *commit, gchar const *tree, gchar const *comment, gboolean signoff, gboolean amend, gchar **ref, GError **error)
 {
 	gchar *fullcomment;
 	
@@ -733,21 +795,47 @@ commit_tree(GitgCommit *commit, gchar const *tree, gchar const *comment, gboolea
 		fullcomment = g_strdup(comment);
 	}
 	
-	gchar *head = gitg_repository_parse_ref(commit->priv->repository, "HEAD");
-
-	gchar **lines = gitg_repository_command_with_input_and_outputv(commit->priv->repository, fullcomment, error, "commit-tree", tree, head ? "-p" : NULL, head, NULL);
+	gchar *head;
+	
+	if (amend)
+	{
+		head = gitg_repository_parse_ref(commit->priv->repository, "HEAD^");
+	}
+	else
+	{
+		head = gitg_repository_parse_ref(commit->priv->repository, "HEAD");
+	}
+	
+	GitgRunner *runner = gitg_runner_new_synchronized (1000);
+	GString *buffer = g_string_new ("");
+	
+	if (amend)
+	{
+		set_amend_environment (commit, runner);
+	}
+	
+	g_signal_connect (runner, "update", G_CALLBACK (on_commit_tree_update), buffer);
+	gitg_repository_run_command_with_inputv (commit->priv->repository,
+	                                         runner,
+	                                         fullcomment,
+	                                         error,
+	                                         "commit-tree",
+	                                         tree,
+	                                         head ? "-p" : NULL,
+	                                         head,
+	                                         NULL);
 
 	g_free(head);
 	g_free(fullcomment);
+	g_object_unref (runner);
 
-	if (!lines || strlen(*lines) != HASH_SHA_SIZE)
+	if (buffer->len != HASH_SHA_SIZE)
 	{
-		g_strfreev(lines);
+		g_string_free(buffer, TRUE);
 		return FALSE;
 	}
 	
-	*ref = g_strdup(*lines);
-	g_strfreev(lines);
+	*ref = g_string_free (buffer, FALSE);
 	return TRUE;
 }
 
@@ -759,16 +847,24 @@ update_ref(GitgCommit *commit, gchar const *ref, gchar const *subject, GError **
 }
 
 gboolean
-gitg_commit_commit(GitgCommit *commit, gchar const *comment, gboolean signoff, GError **error)
+gitg_commit_commit(GitgCommit *commit, gchar const *comment, gboolean signoff, gboolean amend, GError **error)
 {
 	g_return_val_if_fail(GITG_IS_COMMIT(commit), FALSE);
 	
 	gchar *tree;
+
 	if (!write_tree(commit, &tree, error))
 		return FALSE;
 	
+	gchar *path = g_build_filename (gitg_repository_get_path (commit->priv->repository),
+	                                ".git",
+	                                "COMMIT_EDITMSG",
+	                                NULL);
+	g_file_set_contents (path, comment, -1, NULL);\
+	g_free (path);
+	
 	gchar *ref;
-	gboolean ret = commit_tree(commit, tree, comment, signoff, &ref, error);
+	gboolean ret = commit_tree(commit, tree, comment, signoff, amend, &ref, error);
 	g_free(tree);
 	
 	if (!ret)
@@ -810,7 +906,8 @@ gitg_commit_revert(GitgCommit *commit, GitgChangedFile *file, gchar const *hunk,
 		
 		g_free(path);
 		
-		remove_file(commit, file);
+		update_index_file(commit, file);
+		update_index_unstaged(commit, file);
 		g_object_unref(f);
 	}
 	else
@@ -869,3 +966,75 @@ on_changed_file_changed(GitgChangedFile *file, GitgCommit *commit)
 {
 	refresh_changes(commit, file);
 }
+
+GitgChangedFile *
+gitg_commit_find_changed_file(GitgCommit *commit, GFile *file)
+{
+	g_return_val_if_fail(GITG_IS_COMMIT(commit), NULL);
+	g_return_val_if_fail(G_IS_FILE(file), NULL);
+	
+	GitgChangedFile *f = g_hash_table_lookup(commit->priv->files, file);
+	
+	if (f != NULL)
+	{
+		return g_object_ref(f);
+	}
+	else
+	{
+		return NULL;
+	}
+}
+
+gchar *
+gitg_commit_amend_message (GitgCommit *commit)
+{
+	g_return_val_if_fail (GITG_IS_COMMIT (commit), NULL);
+	
+	gchar **out;
+	
+	out = gitg_repository_command_with_outputv (commit->priv->repository,
+	                                            NULL,
+	                                            "cat-file",
+	                                            "commit",
+	                                            "HEAD",
+	                                            NULL);
+
+	gchar *ret = NULL;
+	
+	if (out)
+	{
+		gchar **ptr = out;
+		
+		while (*ptr)
+		{
+			if (!**ptr)
+			{
+				++ptr;
+				break;
+			}
+			
+			++ptr;
+		}
+		
+		if (*ptr && **ptr)
+		{
+			GString *buffer = g_string_new ("");
+			
+			while (*ptr)
+			{
+				if (buffer->len != 0)
+				{
+					g_string_append_c (buffer, '\n');
+				}
+				
+				g_string_append (buffer, *ptr);
+				++ptr;
+			}
+			
+			ret = g_string_free (buffer, FALSE);
+		}
+	}
+	
+	g_strfreev (out);
+	return ret;
+}
