@@ -499,19 +499,27 @@ on_diff_files_update(GitgRunner *runner, gchar **buffer, GitgRevisionView *self)
 		if (**line == '\0')
 			continue;
 		
-		gchar **parts = g_strsplit(*line, " ", 5);
+		// Count parents
+		gint parents = 0;
+		gchar *ptr = *line;
 		
-		if (g_strv_length(parts) == 5)
+		while (*(ptr++) == ':')
+			++parents;
+		
+		gint numparts = 3 + 2 * parents;
+		gchar **parts = g_strsplit(ptr, " ", numparts);
+		
+		if (g_strv_length(parts) == numparts)
 		{
-			gchar **files = g_strsplit(parts[4], "\t", -1);
-			DiffFile *f = diff_file_new(parts[2], parts[3], files[0], files[1]);
+			gchar **files = g_strsplit(parts[numparts - 1], "\t", -1);
+
+			DiffFile *f = diff_file_new(parts[parents + 1], parts[numparts - 2], files[0], files[1]);
 			
 			add_diff_file(self, f);
 			diff_file_unref(f);
 
 			g_strfreev(files);
 		}
-		else
 
 		g_strfreev(parts);
 	}
@@ -541,7 +549,7 @@ on_diff_end_loading(GitgRunner *runner, gboolean cancelled, GitgRevisionView *se
 		const gchar *cached = NULL;
 		
 		if (sign == 't')
-			cached == "--cached";
+			cached = "--cached";
 
 		gitg_repository_run_commandv(self->priv->repository, self->priv->diff_files_runner, NULL,
 									"diff-index", "--raw", "-M", "--abbrev=40", head, cached, NULL);
@@ -691,7 +699,9 @@ update_parents(GitgRevisionView *self, GitgRevision *revision)
 
 static void
 update_diff(GitgRevisionView *self, GitgRepository *repository)
-{	
+{
+	GtkTreeSelection *selection;
+	
 	// First cancel a possibly still running diff
 	gitg_runner_cancel(self->priv->diff_runner);
 	gitg_runner_cancel(self->priv->diff_files_runner);
@@ -702,8 +712,17 @@ update_diff(GitgRevisionView *self, GitgRepository *repository)
 	GtkTextBuffer *buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(self->priv->diff));
 	gtk_text_buffer_set_text(buffer, "", 0);
 	
+	selection = gtk_tree_view_get_selection(self->priv->diff_files);
+	g_signal_handlers_block_by_func(selection,
+	                                G_CALLBACK(on_diff_files_selection_changed),
+	                                self);
+
 	gtk_list_store_clear(self->priv->list_store_diff_files);
 	
+	g_signal_handlers_unblock_by_func(selection,
+	                                  G_CALLBACK(on_diff_files_selection_changed),
+	                                  self);
+	
 	if (!self->priv->revision)
 		return;
 
